import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { AdProps, CreateAdProps } from './ad.types';
import { MatcherAdCreatedDomainEvent } from './events/matcher-ad-created.domain-event';

export class AdEntity extends AggregateRoot<AdProps> {
  protected readonly _id: AggregateID;

  static create = (create: CreateAdProps): AdEntity => {
    const props: AdProps = { ...create };
    const ad = new AdEntity({ id: create.id, props });
    ad.addEvent(
      new MatcherAdCreatedDomainEvent({
        metadata: {
          correlationId: create.id,
          timestamp: Date.now(),
        },
        aggregateId: create.id,
        driverDistance: create.driverDistance,
        driverDuration: create.driverDuration,
        passengerDistance: create.passengerDistance,
        passengerDuration: create.passengerDuration,
        fwdAzimuth: create.fwdAzimuth,
        backAzimuth: create.backAzimuth,
      }),
    );
    return ad;
  };

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
