import { Test, TestingModule } from '@nestjs/testing';
import { AD_MESSAGE_PUBLISHER } from '@modules/ad/ad.di-tokens';
import { MATCHER_AD_CREATED_ROUTING_KEY } from '@src/app.constants';
import { PublishMessageWhenMatcherAdIsCreatedDomainEventHandler } from '@modules/ad/core/application/event-handlers/publish-message-when-matcher-ad-is-created.domain-event-handler';
import { MatcherAdCreatedDomainEvent } from '@modules/ad/core/domain/events/matcher-ad-created.domain-event';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('Publish message when matcher ad is created domain event handler', () => {
  let publishMessageWhenMatcherAdIsCreatedDomainEventHandler: PublishMessageWhenMatcherAdIsCreatedDomainEventHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AD_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        PublishMessageWhenMatcherAdIsCreatedDomainEventHandler,
      ],
    }).compile();

    publishMessageWhenMatcherAdIsCreatedDomainEventHandler =
      module.get<PublishMessageWhenMatcherAdIsCreatedDomainEventHandler>(
        PublishMessageWhenMatcherAdIsCreatedDomainEventHandler,
      );
  });

  it('should publish a message', () => {
    jest.spyOn(mockMessagePublisher, 'publish');
    const matcherAdCreatedDomainEvent: MatcherAdCreatedDomainEvent = {
      id: 'some-domain-event-id',
      aggregateId: 'some-aggregate-id',
      metadata: {
        timestamp: new Date('2023-06-28T05:00:00Z').getTime(),
        correlationId: 'some-correlation-id',
      },
      driverDistance: 65845,
      driverDuration: 3254,
      fwdAzimuth: 90,
      backAzimuth: 270,
    };
    publishMessageWhenMatcherAdIsCreatedDomainEventHandler.handle(
      matcherAdCreatedDomainEvent,
    );
    expect(
      publishMessageWhenMatcherAdIsCreatedDomainEventHandler,
    ).toBeDefined();
    expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
      MATCHER_AD_CREATED_ROUTING_KEY,
      '{"id":"some-aggregate-id","metadata":{"correlationId":"some-correlation-id","timestamp":1687928400000},"driverDuration":3254,"driverDistance":65845,"fwdAzimuth":90,"backAzimuth":270}',
    );
  });
});
